/**
 * Thanks for following contributor of codes
 * https://gist.github.com/1866474
 * http://paulirish.com/2011/requestanimationframe-for-smart-animating/
 * http://my.opera.com/emoller/blog/2011/12/20/requestanimationframe-for-smart-er-animating
 * https://github.com/Financial-Times/polyfill-library/blob/master/polyfills/requestAnimationFrame/polyfill.js
**/
let uId = 1;
const uniqueId = () => uId++;
// We use `self` instead of `window` for `WebWorker` support.
const root = (typeof self === 'object' && self.self == self) ? self
    : (typeof global === 'object' && global.global == global) ? global
        : {};
const nowOffset = Date.now();
// use performance api if exist, otherwise use Date.now.
// Date.now polyfill required.
const pnow = () => {
    if (root.performance && typeof root.performance.now === 'function') {
        return root.performance.now();
    }
    // fallback
    return Date.now() - nowOffset;
};
let reservedCBs = {};
let lastTime = Date.now();
const polyfillRaf = (callback) => {
    if (typeof callback !== 'function') {
        throw new TypeError(callback + ' is not a function');
    }
    const currentTime = Date.now();
    const gap = currentTime - lastTime;
    const delay = gap > 16 ? 0 : 16 - gap;
    const id = uniqueId();
    reservedCBs[id] = callback;
    // keys(reservedCBs).length > 1 의미는 이미 setTimeout 이 걸려있는 경우.
    // 함께 callback 이 실행될 수 있게 reservedCBs 에만 추가해주고 return
    if (Object.keys(reservedCBs).length > 1)
        return id;
    setTimeout(() => {
        lastTime = currentTime;
        const copied = reservedCBs;
        reservedCBs = {};
        Object.keys(copied)
            .forEach((key) => copied[key](pnow()));
    }, delay);
    return id;
};
const polyfillCaf = (id) => {
    delete reservedCBs[id];
};
const vendorPrefixes = ['', 'webkit', 'moz', 'ms', 'o'];
const getRequestAnimationFrame = (vp) => {
    if (typeof vp !== 'string')
        return polyfillRaf;
    if (vp === '')
        return root['requestAnimationFrame'];
    return root[vp + 'RequestAnimationFrame'];
};
const getCancelAnimationFrame = (vp) => {
    if (typeof vp !== 'string')
        return polyfillCaf;
    if (vp === '')
        return root['cancelAnimationFrame'];
    return root[vp + 'CancelAnimationFrame'] || root[vp + 'CancelRequestAnimationFrame'];
};
const find = (arr, predicate) => {
    let i = 0;
    while (arr[i] !== void 0) {
        if (predicate(arr[i]))
            return arr[i];
        i = i + 1;
    }
};
const vp = find(vendorPrefixes, (vp) => !!getRequestAnimationFrame(vp));
const raf = getRequestAnimationFrame(vp);
const caf = getCancelAnimationFrame(vp);

// References:
// Browser export as a global
root.requestAnimationFrame = raf;
root.cancelAnimationFrame = caf;
// Node Export
if (typeof module !== "undefined" && module.exports) {
    module.exports = {
        requestAnimationFrame: raf,
        cancelAnimationFrame: caf,
    };
}
var index = { requestAnimationFrame: raf, cancelAnimationFrame: caf };
/*
 * Below 4 methods should be available on consumer side:
 *
 * <script src="https://.../index.umd.min.js"></script>
 * requestAnimationFrame, cancelAnimationFrame
 *
 * const {requestAnimationFrame, cancelAnimationFrame} require('request-animation-frame-polyfill')
 * const rafp require('request-animation-frame-polyfill')
 * rafp.requestAnimationFrame, rafp.cancelAnimationFrame
 *
 * import rafp from 'request-animation-frame-polyfill'
 * rafp.requestAnimationFrame, rafp.cancelAnimationFrame
 *
 * import {requestAnimationFrame, cancelAnimationFrame} from 'request-animation-frame-polyfill'
 * import * as rafp from 'request-animation-frame-polyfill'
 * rafp.requestAnimationFrame, rafp.cancelAnimationFrame
*/

export default index;
export { caf as cancelAnimationFrame, raf as requestAnimationFrame };
